@@ -21,31 +21,36 @@ from typing import (
 )
 from uuid import UUID
 
-from esmerald.backgound import BackgroundTask, BackgroundTasks
-from esmerald.datastructures import ResponseContainer
-from esmerald.enums import MediaType
-from esmerald.exceptions import ImproperlyConfigured
-from esmerald.injector import Inject
-from esmerald.kwargs import KwargsModel
-from esmerald.permissions.utils import continue_or_raise_permission_exception
-from esmerald.requests import Request
-from esmerald.responses import ORJSONResponse, Response, UJSONResponse
-from esmerald.routing.views import APIView
-from esmerald.signature import SignatureModelFactory, get_signature_model
-from esmerald.typing import Void
-from esmerald.utils.helpers import is_async_callable, is_class_and_subclass
-from esmerald.utils.sync import AsyncCallable
 from starlette.convertors import CONVERTOR_TYPES
 from starlette.requests import HTTPConnection
 from starlette.responses import JSONResponse
 from starlette.responses import Response as StarletteResponse
 from starlette.routing import Mount as Mount  # noqa
 from starlette.routing import compile_path
-from starlette.types import Scope
+from starlette.types import Receive, Scope, Send
 from typing_extensions import TypedDict
 
+from esmerald.backgound import BackgroundTask, BackgroundTasks
+from esmerald.datastructures import ResponseContainer
+from esmerald.enums import MediaType
+from esmerald.exceptions import ImproperlyConfigured
+from esmerald.injector import Inject
+from esmerald.permissions.utils import continue_or_raise_permission_exception
+from esmerald.requests import Request
+from esmerald.responses import ORJSONResponse, Response, UJSONResponse
+from esmerald.routing.views import APIView
+from esmerald.transformers.model import TransformerModel
+from esmerald.transformers.signature import SignatureFactory
+from esmerald.transformers.utils import get_signature
+from esmerald.typing import Void
+from esmerald.utils.helpers import is_async_callable, is_class_and_subclass
+from esmerald.utils.sync import AsyncCallable
+
 if TYPE_CHECKING:
+    from pydantic.typing import AnyCallable
+
     from esmerald.applications import Esmerald
+    from esmerald.interceptors.types import Interceptor
     from esmerald.permissions.types import Permission
     from esmerald.routing.router import HTTPHandler, WebSocketHandler
     from esmerald.types import (
@@ -56,7 +61,6 @@ if TYPE_CHECKING:
         ResponseHeaders,
         ResponseType,
     )
-    from pydantic.typing import AnyCallable
 
 param_type_map = {
     "str": str,
@@ -113,31 +117,31 @@ class BaseSignature:
         Websockets do not support methods.
         """
         if not self.signature_model:
-            self.signature_model = SignatureModelFactory(
+            self.signature_model = SignatureFactory(
                 fn=cast("AnyCallable", self.fn),
                 dependency_names=self.dependency_names,
-            ).create_signature_model()
+            ).create_signature()
 
         for dependency in list(self.get_dependencies().values()):
             if not dependency.signature_model:
-                dependency.signature_model = SignatureModelFactory(
+                dependency.signature_model = SignatureFactory(
                     fn=dependency.dependency, dependency_names=self.dependency_names
-                ).create_signature_model()
+                ).create_signature()
 
-        kwargs_model = self.create_handler_kwargs_model()
+        transformer_model = self.create_handler_transformer_model()
         if not is_websocket:
-            self.kwargs = kwargs_model
+            self.transformer = transformer_model
             for method in self.methods:
-                self.route_map[method] = (self, kwargs_model)
+                self.route_map[method] = (self, transformer_model)
         else:
-            self.websocket_parameter_model = kwargs_model
+            self.websocket_parameter_model = transformer_model
 
-    def create_handler_kwargs_model(self) -> "KwargsModel":
-        """Method to create a KwargsModel for a given handler."""
+    def create_handler_transformer_model(self) -> "TransformerModel":
+        """Method to create a TransformerModel for a given handler."""
         dependencies = self.get_dependencies()
-        signature_model = get_signature_model(self)
+        signature_model = get_signature(self)
 
-        return KwargsModel.create_for_signature_model(
+        return TransformerModel.create_signature(
             signature_model=signature_model,
             dependencies=dependencies,
             path_parameters=self.path_parameters,
@@ -149,7 +153,7 @@ class BaseResponseHandler:
     In charge of handling the responses of the handlers.
     """
 
-    def create_response_container_handler(
+    def response_container_handler(
         self,
         cookies: "ResponseCookies",
         headers: Dict[str, Any],
@@ -175,7 +179,7 @@ class BaseResponseHandler:
 
         return response_content
 
-    def create_response_handler(
+    def response_handler(
         self,
         cookies: "ResponseCookies",
         headers: Optional["ResponseHeaders"] = None,
@@ -205,19 +209,34 @@ class BaseResponseHandler:
 
         return response_content
 
-    def create_json_response_handler(
-        self, status_code: Optional[int] = None
+    def json_response_handler(
+        self,
+        status_code: Optional[int] = None,
+        cookies: Optional["ResponseCookies"] = None,
+        headers: Optional["ResponseHeaders"] = None,
     ) -> "AsyncAnyCallable":
         """Creates a handler function for Esmerald JSON responses"""
 
         async def response_content(data: Response, **kwargs: Dict[str, Any]) -> StarletteResponse:
+            _cookies = self.get_cookies(cookies, [])
+            _headers = {
+                **self.get_headers(headers),
+                **data.headers,
+                **self.allow_header,
+            }
+            for cookie in _cookies:
+                data.set_cookie(**cookie)
+
+            for header, value in _headers.items():
+                data.headers[header] = value
+
             if status_code:
                 data.status_code = status_code
             return data
 
         return response_content
 
-    def create_starlette_response_handler(
+    def starlette_response_handler(
         self,
         cookies: "ResponseCookies",
         headers: Optional["ResponseHeaders"] = None,
@@ -246,7 +265,7 @@ class BaseResponseHandler:
 
         return response_content
 
-    def create_handler(
+    def handler(
         self,
         background: Optional[Union["BackgroundTask", "BackgroundTasks"]],
         cookies: "ResponseCookies",
@@ -284,7 +303,7 @@ class BaseResponseHandler:
         scope: "Scope",
         request: Request,
         route: Union["HTTPHandler", "WebSocketHandler"],
-        parameter_model: "KwargsModel",
+        parameter_model: "TransformerModel",
     ) -> "StarletteResponse":
         response: Optional["StarletteResponse"] = None
         if not response:
@@ -302,7 +321,7 @@ class BaseResponseHandler:
         scope: "Scope",
         request: Request,
         route: Union["HTTPHandler", "WebSocketHandler"],
-        parameter_model: "KwargsModel",
+        parameter_model: "TransformerModel",
     ) -> "StarletteResponse":
         response_data = None
 
@@ -320,7 +339,7 @@ class BaseResponseHandler:
 
     @staticmethod
     async def _get_response_data(
-        route: "HTTPHandler", parameter_model: "KwargsModel", request: Request
+        route: "HTTPHandler", parameter_model: "TransformerModel", request: Request
     ) -> Any:
         """
         Determines what kwargs are required for the given handler and assignes to the specific
@@ -328,18 +347,18 @@ class BaseResponseHandler:
 
         It supports more one object payload to be sent.
         """
-        signature_model = get_signature_model(route)
+        signature_model = get_signature(route)
 
         if parameter_model.has_kwargs:
             kwargs = parameter_model.to_kwargs(connection=request)
             request_data = kwargs.get("data")
             if request_data:
                 kwargs["data"] = await request_data
-            for dependency in parameter_model.expected_dependencies:
-                kwargs[dependency.key] = await parameter_model.resolve_dependency(
+            for dependency in parameter_model.dependencies:
+                kwargs[dependency.key] = await parameter_model.get_dependencies(
                     dependency=dependency, connection=request, **kwargs
                 )
-            parsed_kwargs = signature_model.parse_values_from_connection_kwargs(
+            parsed_kwargs = signature_model.parse_values_for_connection(
                 connection=request, **kwargs
             )
         else:
@@ -355,7 +374,6 @@ class BaseResponseHandler:
 
         if is_async_callable(fn):
             return await fn()
-
         return fn()
 
     def get_response_handler(self) -> Callable[[Any], Awaitable[StarletteResponse]]:
@@ -373,7 +391,7 @@ class BaseResponseHandler:
             cookies = self.get_response_cookies()
 
             if is_class_and_subclass(self.signature.return_annotation, ResponseContainer):
-                handler = self.create_response_container_handler(
+                handler = self.response_container_handler(
                     cookies=cookies,
                     media_type=self.media_type,
                     status_code=self.status_code,
@@ -383,22 +401,24 @@ class BaseResponseHandler:
                 self.signature.return_annotation,
                 (JSONResponse, ORJSONResponse, UJSONResponse),
             ):
-                handler = self.create_json_response_handler(status_code=self.status_code)
+                handler = self.json_response_handler(
+                    status_code=self.status_code, cookies=cookies, headers=headers
+                )
             elif is_class_and_subclass(self.signature.return_annotation, Response):
-                handler = self.create_response_handler(
+                handler = self.response_handler(
                     cookies=cookies,
                     status_code=self.status_code,
                     media_type=self.media_type,
                     headers=headers,
                 )
             elif is_class_and_subclass(self.signature.return_annotation, StarletteResponse):
-                handler = self.create_starlette_response_handler(
+                handler = self.starlette_response_handler(
                     cookies=cookies,
                     media_type=self.media_type,
                     headers=headers,
                 )
             else:
-                handler = self.create_handler(
+                handler = self.handler(
                     background=self.background,
                     cookies=cookies,
                     headers=headers,
@@ -478,7 +498,7 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
         level_dependencies = (level.dependencies or {} for level in self.parent_levels)
         return {name for level in level_dependencies for name in level.keys()}
 
-    def resolve_permissions(self) -> List["Permission"]:
+    def get_permissions(self) -> List["Permission"]:
         """
         Returns all the permissions in the handler scope from the ownsership layers.
         """
@@ -504,7 +524,7 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
             self._dependencies = {}
             for level in self.parent_levels:
                 for key, value in (level.dependencies or {}).items():
-                    self.has_dependency_unique(
+                    self.is_unique_dependency(
                         dependencies=self._dependencies,
                         key=key,
                         injector=value,
@@ -513,7 +533,7 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
         return cast("Dict[str, Inject]", self._dependencies)
 
     @staticmethod
-    def has_dependency_unique(dependencies: Dict[str, Inject], key: str, injector: Inject) -> None:
+    def is_unique_dependency(dependencies: Dict[str, Inject], key: str, injector: Inject) -> None:
         """
         Validates that a given inject has not been already defined under a
         different key in any of the levels.
@@ -572,7 +592,32 @@ class BaseHandlerMixin(BaseSignature, BaseResponseHandler, OpenAPIDefinitionMixi
 
         Raises a PermissionDenied exception if not allowed..
         """
-        for permission in self.resolve_permissions():
+        for permission in self.get_permissions():
             permission = await permission()
             permission.has_permission(connection, self)
             continue_or_raise_permission_exception(connection, self, permission)
+
+
+class BaseInterceptorMixin(BaseHandlerMixin):
+    def get_interceptors(self) -> List["Interceptor"]:
+        """
+        Returns all the interceptors in the handler scope from the ownsership layers.
+        """
+        if self._interceptors is Void:
+            self._interceptors = []
+            for layer in self.parent_levels:
+                self._interceptors.extend(layer.interceptors or [])
+            self._interceptors = cast(
+                "List[Interceptor]",
+                [AsyncCallable(interceptors) for interceptors in self._interceptors],
+            )
+        return cast("List[Interceptor]", self._interceptors)
+
+    async def intercept(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
+        """
+        Checks for every interceptor on each level and runs them all before reaching any
+        of the handlers.
+        """
+        for interceptor in self.get_interceptors():
+            interceptor = await interceptor()
+            await interceptor.intercept(scope, receive, send)
