@@ -19,30 +19,6 @@ from typing import (
     cast,
 )
 
-from esmerald.conf import settings
-from esmerald.datastructures import File, Redirect, URLPath
-from esmerald.enums import HttpMethod, MediaType
-from esmerald.exceptions import (
-    ImproperlyConfigured,
-    MethodNotAllowed,
-    NotFound,
-    ValidationErrorException,
-)
-from esmerald.injector import Inject
-from esmerald.kwargs import KwargsModel
-from esmerald.openapi.datastructures import ResponseSpec
-from esmerald.requests import Request
-from esmerald.responses import Response
-from esmerald.routing.base import BaseHandlerMixin
-from esmerald.routing.gateways import Gateway, WebSocketGateway
-from esmerald.routing.views import APIView
-from esmerald.signature import SignatureModel, get_signature_model
-from esmerald.typing import Void, VoidType
-from esmerald.urls import include
-from esmerald.utils.constants import DATA, REDIRECT_STATUS_CODES, REQUEST, SOCKET
-from esmerald.utils.helpers import is_async_callable, is_class_and_subclass
-from esmerald.utils.url import clean_path
-from esmerald.websockets import WebSocket, WebSocketClose
 from starlette import status
 from starlette.requests import HTTPConnection
 from starlette.responses import JSONResponse
@@ -55,7 +31,37 @@ from starlette.routing import WebSocketRoute as StarletteWebSocketRoute
 from starlette.routing import compile_path
 from starlette.types import ASGIApp, Receive, Scope, Send
 
+from esmerald.conf import settings
+from esmerald.datastructures import File, Redirect, URLPath
+from esmerald.enums import HttpMethod, MediaType
+from esmerald.exceptions import (
+    ImproperlyConfigured,
+    MethodNotAllowed,
+    NotFound,
+    ValidationErrorException,
+)
+from esmerald.injector import Inject
+from esmerald.interceptors.types import Interceptor
+from esmerald.openapi.datastructures import ResponseSpecification
+from esmerald.requests import Request
+from esmerald.responses import Response
+from esmerald.routing.base import BaseHandlerMixin
+from esmerald.routing.gateways import Gateway, WebSocketGateway
+from esmerald.routing.views import APIView
+from esmerald.transformers.datastructures import EsmeraldSignature as SignatureModel
+from esmerald.transformers.model import TransformerModel
+from esmerald.transformers.utils import get_signature
+from esmerald.typing import Void, VoidType
+from esmerald.urls import include
+from esmerald.utils.constants import DATA, REDIRECT_STATUS_CODES, REQUEST, SOCKET
+from esmerald.utils.helpers import is_async_callable, is_class_and_subclass
+from esmerald.utils.url import clean_path
+from esmerald.websockets import WebSocket, WebSocketClose
+
 if TYPE_CHECKING:
+    from openapi_schemas_pydantic.v3_1_0 import SecurityRequirement
+    from pydantic.typing import AnyCallable
+
     from esmerald.applications import Esmerald
     from esmerald.exceptions import HTTPException
     from esmerald.permissions.types import Permission
@@ -76,8 +82,6 @@ if TYPE_CHECKING:
         ResponseType,
         RouteParent,
     )
-    from openapi_schemas_pydantic.v3_1_0 import SecurityRequirement
-    from pydantic.typing import AnyCallable
 
 
 class Parent:
@@ -155,6 +159,7 @@ class Router(StarletteRouter, Parent):
         "name",
         "dependencies",
         "exception_handlers",
+        "interceptors",
         "permissions",
         "middleware",
         "response_class",
@@ -179,6 +184,7 @@ class Router(StarletteRouter, Parent):
         name: Optional[str] = None,
         dependencies: Optional["Dependencies"] = None,
         exception_handlers: Optional["ExceptionHandlers"] = None,
+        interceptors: Optional[List["Interceptor"]] = None,
         permissions: Optional[List["Permission"]] = None,
         middleware: Optional[List["Middleware"]] = None,
         response_class: Optional["ResponseType"] = None,
@@ -225,6 +231,7 @@ class Router(StarletteRouter, Parent):
         self.parent: Optional["Router"] = parent or self.app
         self.dependencies = dependencies or {}
         self.exception_handlers = exception_handlers or {}
+        self.interceptors = interceptors or []
         self.permissions = permissions or []
         self.middleware = middleware or []
         self.routes = routes
@@ -286,6 +293,7 @@ class Router(StarletteRouter, Parent):
         handler: "HTTPHandler",
         dependencies: Optional["Dependencies"] = None,
         exception_handlers: Optional["ExceptionHandlers"] = None,
+        interceptors: Optional[List["Interceptor"]] = None,
         permissions: Optional[List["Permission"]] = None,
         middleware: Optional[List["Middleware"]] = None,
         name: Optional[str] = None,
@@ -308,6 +316,7 @@ class Router(StarletteRouter, Parent):
             include_in_schema=include_in_schema,
             dependencies=dependencies,
             exception_handlers=exception_handlers,
+            interceptors=interceptors,
             permissions=permissions,
             middleware=middleware,
             deprecated=deprecated,
@@ -323,6 +332,7 @@ class Router(StarletteRouter, Parent):
         name: Optional[str] = None,
         dependencies: Optional["Dependencies"] = None,
         exception_handlers: Optional["ExceptionHandlers"] = None,
+        interceptors: Optional[List["Interceptor"]] = None,
         permissions: Optional[List["Permission"]] = None,
         middleware: Optional[List["Middleware"]] = None,
     ) -> None:
@@ -337,6 +347,7 @@ class Router(StarletteRouter, Parent):
             name=name,
             dependencies=dependencies,
             exception_handlers=exception_handlers,
+            interceptors=interceptors,
             permissions=permissions,
             middleware=middleware,
         )
@@ -432,7 +443,7 @@ class HTTPHandler(BaseHandlerMixin, StarletteRoute):
         tags: Optional[List[Union[str, Enum]]] = None,
         deprecated: Optional[bool] = None,
         response_description: Optional[str] = "Successful Response",
-        responses: Optional[Dict[int, ResponseSpec]] = None,
+        responses: Optional[Dict[int, ResponseSpecification]] = None,
         security: Optional[List["SecurityRequirement"]] = None,
         operation_id: Optional[str] = None,
         raise_exceptions: Optional[List[Type["HTTPException"]]] = None,
@@ -464,15 +475,15 @@ class HTTPHandler(BaseHandlerMixin, StarletteRoute):
         self.operation_id = operation_id
 
         if not methods:
-            methods = [HttpMethod.GET]
+            methods = [HttpMethod.GET.value]
 
         for method in methods or []:
             if not isinstance(method, str):
                 raise ImproperlyConfigured(f"`{method}` in `methods` must be a string.")
 
-        self.methods: Set[str] = {HttpMethod[method] for method in methods}
+        self.methods: Set[str] = {HttpMethod[method].value for method in methods}
         if HttpMethod.GET in methods:
-            self.methods.add(HttpMethod.HEAD)
+            self.methods.add(HttpMethod.HEAD.value)
 
         if isinstance(status_code, IntEnum):
             status_code = int(status_code)
@@ -491,7 +502,7 @@ class HTTPHandler(BaseHandlerMixin, StarletteRoute):
         self.response_headers = response_headers
         self.background = background
         self.signature_model: Optional[Type["SignatureModel"]] = None
-        self.kwargs: Optional["KwargsModel"] = None
+        self.transformer: Optional["TransformerModel"] = None
         self.response_description = response_description
         self.responses = responses or {}
         self.content_encoding = content_encoding
@@ -500,7 +511,7 @@ class HTTPHandler(BaseHandlerMixin, StarletteRoute):
 
         self.fn: Optional["AnyCallable"] = None
         self.app: Optional["ASGIApp"] = None
-        self.route_map: Dict["HTTPMethod" : Tuple["HTTPHandler", "KwargsModel"]] = {}
+        self.route_map: Dict["HTTPMethod" : Tuple["HTTPHandler", "TransformerModel"]] = {}
         self.path_regex, self.path_format, self.param_convertors = compile_path(path)
 
     def handle_status_code(self):
@@ -598,7 +609,7 @@ class HTTPHandler(BaseHandlerMixin, StarletteRoute):
         request = Request(scope=scope, receive=receive, send=send)
         route_handler, parameter_model = self.route_map[scope["method"]]
 
-        if self.resolve_permissions():
+        if self.get_permissions():
             connection = HTTPConnection(scope=scope, receive=receive)
             await self.allow_connection(connection)
 
@@ -651,8 +662,10 @@ class HTTPHandler(BaseHandlerMixin, StarletteRoute):
                     "Functions decorated with 'route, websocket, get, patch, put, post and delete' must be async functions"
                 )
 
-    def validate_handler(self):
-        self.check_handler_function()
+    def validate_annotations(self):
+        """
+        Validate annotations of the handlers.
+        """
         return_annotation = self.signature.return_annotation
 
         if return_annotation is Signature.empty:
@@ -681,11 +694,22 @@ class HTTPHandler(BaseHandlerMixin, StarletteRoute):
             MediaType.HTML,
         ]:
             self.media_type = MediaType.TEXT
-        if SOCKET in self.signature.parameters:
-            raise ImproperlyConfigured("The 'socket' argument is not supported with http handlers")
+
+    def validate_reserved_kwargs(self):
+        """
+        Validates if special words are in the signature.
+        """
         if DATA in self.signature.parameters and "GET" in self.methods:
             raise ImproperlyConfigured("'data' argument is unsupported for 'GET' request handlers")
 
+        if SOCKET in self.signature.parameters:
+            raise ImproperlyConfigured("The 'socket' argument is not supported with http handlers")
+
+    def validate_handler(self):
+        self.check_handler_function()
+        self.validate_annotations()
+        self.validate_reserved_kwargs()
+
     async def to_response(self, app: "Esmerald", data: Any) -> StarletteResponse:
         response_handler = self.get_response_handler()
         return await response_handler(app=app, data=data)
@@ -730,7 +754,7 @@ class WebSocketHandler(BaseHandlerMixin, StarletteWebSocketRoute):
         self.permissions = permissions
         self.middleware = middleware
         self.signature_model: Optional[Type["SignatureModel"]] = None
-        self.websocket_parameter_model: Optional["KwargsModel"] = None
+        self.websocket_parameter_model: Optional["TransformerModel"] = None
 
     def __call__(self, fn: "AnyCallable") -> "ASGIApp":
         self.fn = fn
@@ -738,19 +762,10 @@ class WebSocketHandler(BaseHandlerMixin, StarletteWebSocketRoute):
         self.validate_websocket_handler_function()
         return self
 
-    def validate_websocket_handler_function(self) -> None:
+    def validate_reserved_words(self, signature: "Signature"):
         """
-        Validates the route handler function once it is set by inspecting its
-        return annotations.
+        Validates if special words are in the signature.
         """
-        if not self.fn:
-            raise ImproperlyConfigured(
-                "Cannot call check_handler_function without first setting self.fn"
-            )
-
-        fn = cast("AnyCallable", self.fn)
-        signature = Signature.from_callable(fn)
-
         if signature.return_annotation is not None:
             raise ImproperlyConfigured("Websocket functions should return 'None'.")
 
@@ -761,9 +776,21 @@ class WebSocketHandler(BaseHandlerMixin, StarletteWebSocketRoute):
                     f"The '{kwarg}'is not supported with websocket handlers."
                 )
 
+    def validate_websocket_handler_function(self) -> None:
+        """
+        Validates the route handler function once it is set by inspecting its
+        return annotations.
+        """
+        if not self.fn:
+            raise ImproperlyConfigured(
+                "Cannot call check_handler_function without first setting self.fn"
+            )
+        signature = Signature.from_callable(self.fn)
+        self.validate_reserved_words(signature=signature)
+
         if SOCKET not in signature.parameters:
             raise ImproperlyConfigured("Websocket handlers must set a 'socket' argument.")
-        if not is_async_callable(fn):
+        if not is_async_callable(self.fn):
             raise ImproperlyConfigured(
                 "Functions decorated with 'asgi, get, patch, put, post and delete' must be async functions."
             )
@@ -771,7 +798,7 @@ class WebSocketHandler(BaseHandlerMixin, StarletteWebSocketRoute):
     async def handle(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
         """The handle of a websocket"""
         websocket = WebSocket[Any, Any](scope=scope, receive=receive, send=send)
-        if self.resolve_permissions():
+        if self.get_permissions():
             await self.allow_connection(connection=websocket)
 
         kwargs = await self.get_kwargs(websocket=websocket)
@@ -793,13 +820,13 @@ class WebSocketHandler(BaseHandlerMixin, StarletteWebSocketRoute):
         """
         assert self.websocket_parameter_model, "handler parameter model not defined."
 
-        signature_model = get_signature_model(self)
+        signature_model = get_signature(self)
         kwargs = self.websocket_parameter_model.to_kwargs(connection=websocket)
-        for dependency in self.websocket_parameter_model.expected_dependencies:
-            kwargs[dependency.key] = await self.websocket_parameter_model.resolve_dependency(
+        for dependency in self.websocket_parameter_model.dependencies:
+            kwargs[dependency.key] = await self.websocket_parameter_model.get_dependencies(
                 dependency=dependency, connection=websocket, **kwargs
             )
-        return signature_model.parse_values_from_connection_kwargs(connection=websocket, **kwargs)
+        return signature_model.parse_values_for_connection(connection=websocket, **kwargs)
 
 
 class Include(Mount):
@@ -824,6 +851,7 @@ class Include(Mount):
         "name",
         "dependencies",
         "exception_handlers",
+        "interceptors",
         "permissions",
         "middleware",
         "deprecated",
@@ -841,6 +869,7 @@ class Include(Mount):
         parent: Optional["Router"] = None,
         dependencies: Optional["Dependencies"] = None,
         exception_handlers: Optional[Dict[Union[int, Type[Exception]], "ExceptionHandler"]] = None,
+        interceptors: Optional[List["Interceptor"]] = None,
         permissions: Optional[List["Permission"]] = None,
         middleware: Optional[List["Middleware"]] = None,
         include_in_schema: Optional[bool] = True,
@@ -872,6 +901,7 @@ class Include(Mount):
         self.pattern = pattern
         self.include_in_schema = include_in_schema
         self.dependencies = dependencies or {}
+        self.interceptors = interceptors or []
         self.permissions = permissions or []
         self.middleware = middleware or []
         self.exception_handlers = exception_handlers or {}
@@ -971,6 +1001,7 @@ class Include(Mount):
                             handler=route_handler,
                             name=route_handler.path,
                             middleware=route.middleware,
+                            interceptors=self.interceptors,
                             permissions=route.permissions,
                             exception_handlers=route.exception_handlers,
                         )
